@@ -17,9 +17,18 @@ import warnings
 from datetime import datetime
 
 import numpy as np
+try:
+    from numpy.exceptions import ComplexWarning
+except ImportError:
+    from numpy import ComplexWarning
+try:
+    from numpy.rec import fromrecords
+except ImportError:
+    from numpy.core.records import fromrecords
 
 from .helpers import AssertRaisesRegexMixin
 from mrcfile import constants
+from mrcfile import dtypes
 from mrcfile.mrcobject import MrcObject
 from mrcfile import utils
 
@@ -150,6 +159,12 @@ class MrcObjectTest(AssertRaisesRegexMixin, unittest.TestCase):
         assert self.mrcobject.extended_header is ext2
         assert self.mrcobject.header.nsymbt == ext2.nbytes
     
+    @unittest.skipIf(sys.maxsize <= np.iinfo(np.int32).max, "can't run test on 32-bit")
+    def test_replacing_extended_header_too_big(self):
+        ext = np.empty((int(np.iinfo(np.int32).max) + 1,), dtype='V1')
+        with self.assertRaisesRegex(ValueError, "extended header is too large"):
+            self.mrcobject.set_extended_header(ext)
+    
     def test_header_is_correct_for_2d_data(self):
         x, y = 3, 2
         data = np.arange(y * x, dtype=np.int16).reshape(y, x)
@@ -268,6 +283,12 @@ class MrcObjectTest(AssertRaisesRegexMixin, unittest.TestCase):
             self.mrcobject.set_data(np.arange(2, dtype=np.int16)
                                     .reshape(1, 1, 1, 1, 2))
     
+    @unittest.skipIf(sys.maxsize <= np.iinfo(np.int32).max, "can't run test on 32-bit")
+    def test_data_array_too_big_raises_exception(self):
+        data = np.empty((int(np.iinfo(np.int32).max) + 1, 1), dtype=np.int8)
+        with self.assertRaisesRegex(ValueError, "data array is too large"):
+            self.mrcobject.set_data(data)
+    
     def assert_dtype_raises_exception(self, data):
         with self.assertRaisesRegex(ValueError, 'dtype'):
             self.mrcobject.set_data(data)
@@ -312,7 +333,7 @@ class MrcObjectTest(AssertRaisesRegexMixin, unittest.TestCase):
         data = np.arange(6, dtype=np.complex64).reshape(3, 2)
         # Suppress complex casting warnings from statistics calculations
         with warnings.catch_warnings():
-            warnings.simplefilter("ignore", np.ComplexWarning)
+            warnings.simplefilter("ignore", ComplexWarning)
             self.mrcobject.set_data(data)
             assert self.mrcobject.data.dtype == np.complex64
             assert self.mrcobject.header.mode == 4
@@ -370,7 +391,7 @@ class MrcObjectTest(AssertRaisesRegexMixin, unittest.TestCase):
         assert utils.byte_orders_equal(header.mode.dtype.byteorder,
                                        orig_byte_order)
         
-        self.mrcobject.set_data(data.newbyteorder())
+        self.mrcobject.set_data(data.view(data.dtype.newbyteorder()))
         assert not utils.byte_orders_equal(header.mode.dtype.byteorder,
                                            orig_byte_order)
         assert header.mode == 2
@@ -511,7 +532,7 @@ class MrcObjectTest(AssertRaisesRegexMixin, unittest.TestCase):
             assert len(w) >= 1, str(len(w)) + " warnings issued"
             assert "Data array contains infinite values" in [str(i.message) for i in w]
             header = self.mrcobject.header
-            assert header.dmin == np.NINF
+            assert header.dmin == -np.inf
             assert header.dmax == np.inf
             assert np.isnan(header.dmean)
             assert np.isnan(header.rms)
@@ -716,6 +737,26 @@ class MrcObjectTest(AssertRaisesRegexMixin, unittest.TestCase):
         print_output = print_stream.getvalue()
         assert "Header field 'ispg' is negative" in print_output
 
+    def test_setting_origin_as_separate_fields(self):
+        self.mrcobject.header.origin.x = 1
+        self.mrcobject.header.origin.y = -5
+        self.mrcobject.header.origin.z = 128
+        expected = fromrecords((1., -5., 128.),
+                               dtype=dtypes.VOXEL_SIZE_DTYPE)
+        np.testing.assert_equal(self.mrcobject.header.origin, expected)
+
+    def test_setting_origin_as_single_field(self):
+        origin = fromrecords((-1., 1., 0.),
+                             dtype=dtypes.VOXEL_SIZE_DTYPE)
+        self.mrcobject.header.origin = origin
+        np.testing.assert_equal(self.mrcobject.header.origin, origin)
+
+    def test_setting_origin_as_single_tuple(self):
+        origin = (-1., 1., 0.)
+        self.mrcobject.header.origin = origin
+        expected = fromrecords(origin, dtype=dtypes.VOXEL_SIZE_DTYPE)
+        np.testing.assert_equal(self.mrcobject.header.origin, expected)
+
 
 if __name__ == '__main__':
     unittest.main()
